#pragma semicolon 1

#include <sourcemod>
#include <sdktools>

#define PLUGIN_VERSION	"1.4"

ConVar mp_bullet_trace;
bool bEnable;

ConVar hRGB;
char sRGB[ 18 ];
ConVar hSpeed;
float fSpeed;
ConVar hDelay;
float fDelay;
ConVar hStartWidth;
float fStartWidth;
ConVar hEndWidth;
float fEndWidth;
	
float g_fClientDelay[MAXPLAYERS+1];

public Plugin:myinfo =
{
	name = "Bullet Trace",
	author = "FrozDark && Edit for new syntax",
	description = "Bullet Trace",
	version = PLUGIN_VERSION,
	url = "http://www.hlmod.ru/"
};

public OnPluginStart()
{
	CreateConVar("sm_bullet_trace_version", PLUGIN_VERSION, "The version of bullet trace", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_CHEAT|FCVAR_DONTRECORD);
	mp_bullet_trace = CreateConVar("mp_bullet_trace", "1", "Whether to enable bullet trace effect on fire", FCVAR_PLUGIN|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_DONTRECORD, true, 0.0, true, 1.0);
	bEnable = mp_bullet_trace.BoolValue;
	HookConVarChange( mp_bullet_trace, OnConVarChanges );
	
	hDelay = CreateConVar("sm_bullet_trace_delay", "0.1", "Delay between next bullet trace", FCVAR_PLUGIN);
	fDelay = hDelay.FloatValue;
	HookConVarChange(hDelay, OnConVarChanges);
	
	hRGB = CreateConVar("sm_bullet_trace_color", "200 200 0", "RGB color of bullet trace", FCVAR_PLUGIN);
	hRGB.GetString(sRGB, sizeof(sRGB));
	HookConVarChange(hRGB, OnConVarChanges);
	
	hSpeed = CreateConVar("sm_bullet_trace_speed", "6000.0", "The speed of bullet trace", FCVAR_PLUGIN, true, 1000.0);
	fSpeed = hSpeed.FloatValue;
	HookConVarChange(hSpeed, OnConVarChanges);
	
	hStartWidth = CreateConVar("sm_bullet_trace_startwidth", "0.7", "The start width of bullet trace", FCVAR_PLUGIN, true, 0.1);
	fStartWidth = hStartWidth.FloatValue;
	HookConVarChange(hStartWidth, OnConVarChanges);
	
	hEndWidth = CreateConVar("sm_bullet_trace_endwidth", "0.2", "The end width of bullet trace", FCVAR_PLUGIN, true, 0.1);
	fEndWidth = hEndWidth.FloatValue;
	HookConVarChange(hEndWidth, OnConVarChanges);
	
	AutoExecConfig(true, "bullet_trace");
	
	HookEvent("bullet_impact", OnBulletImpact);
}

public OnConVarChanges( ConVar convar, const char[  ] oldVal, const char[  ] newVal )
{
	if (convar == mp_bullet_trace)
	{
		bEnable = bool:StringToInt(newVal);
	}
	else if (convar == hDelay)
	{
		fDelay = StringToFloat(newVal);
	}
	else if (convar == hRGB)
	{
		strcopy(sRGB, sizeof(sRGB), newVal);
	}
	else if (convar == hSpeed)
	{
		fSpeed = StringToFloat(newVal);
	}
	else if (convar == hStartWidth)
	{
		fStartWidth = StringToFloat(newVal);
	}
	else if (convar == hEndWidth)
	{
		fEndWidth = StringToFloat(newVal);
	}
}

public OnMapStart()
{
	for (int i = 1; i < sizeof(g_fClientDelay); i++)
	{
		g_fClientDelay[i] = 0.0;
	}
}

public OnClientDisconnect_Post(client)
{
	g_fClientDelay[client] = 0.0;
}

public OnBulletImpact( Event event, const char[  ] name, bool dontBroadcast )
{
	if (!bEnable)
	{
		return;
	}
	
	int client = GetClientOfUserId(event.GetInt("userid"));
	
	float _game_time = GetGameTime();
	
	if (g_fClientDelay[client] > _game_time) return;

	float bulletDestination[3];
	bulletDestination[0] = event.GetFloat("x");
	bulletDestination[1] = event.GetFloat("y");
	bulletDestination[2] = event.GetFloat("z");

	float bulletOrigin[3];
	GetClientEyePosition(client, bulletOrigin);

	float distance = GetVectorDistance(bulletOrigin, bulletDestination);
	float percentage = 0.4 / (distance / 100);
	
	float newBulletOrigin[3];
	newBulletOrigin[0] = bulletOrigin[0] + ((bulletDestination[0] - bulletOrigin[0]) * percentage);
	newBulletOrigin[1] = bulletOrigin[1] + ((bulletDestination[1] - bulletOrigin[1]) * percentage) - 0.08;
	newBulletOrigin[2] = bulletOrigin[2] + ((bulletDestination[2] - bulletOrigin[2]) * percentage);
	
	CreateBulletTrace(newBulletOrigin, bulletDestination, fSpeed, fStartWidth, fEndWidth, sRGB);
	
	int weapon = GetEntPropEnt(client, Prop_Send, "m_hActiveWeapon");
	if (weapon != -1)
	{
		char g_szWeapon[32];
		GetEdictClassname(weapon, g_szWeapon, sizeof(g_szWeapon));
		if (StrEqual(g_szWeapon, "weapon_xm1014", false) || StrEqual(g_szWeapon, "weapon_m3", false))
		{
			return;
		}
	}
	
	g_fClientDelay[client] = _game_time + fDelay;
}

stock CreateBulletTrace(const float origin[ 3 ], const float dest[ 3 ], const float speed = 6000.0, const float startwidth = 0.5, const float endwidth = 0.2, const char[  ] color = "200 200 0")
{
	int entity = CreateEntityByName("env_spritetrail");
	if (entity == -1)
	{
		LogError("Couldn't create entity 'bullet_trace'");
		return -1;
	}
	DispatchKeyValue(entity, "classname", "bullet_trace");
	DispatchKeyValue(entity, "spritename", "materials/sprites/laser.vmt");
	DispatchKeyValue(entity, "renderamt", "255");
	DispatchKeyValue(entity, "rendercolor", color);
	DispatchKeyValue(entity, "rendermode", "5");
	DispatchKeyValueFloat(entity, "startwidth", startwidth);
	DispatchKeyValueFloat(entity, "endwidth", endwidth);
	DispatchKeyValueFloat(entity, "lifetime", 240.0 / speed);
	if (!DispatchSpawn(entity))
	{
		AcceptEntityInput(entity, "Kill");
		LogError("Couldn't create entity 'bullet_trace'");
		return -1;
	}
	
	SetEntPropFloat(entity, Prop_Send, "m_flTextureRes", 0.05);
	
	float vecVeloc[ 3 ], angRotation[ 3 ];
	MakeVectorFromPoints(origin, dest, vecVeloc);
	GetVectorAngles(vecVeloc, angRotation);
	NormalizeVector(vecVeloc, vecVeloc);
	ScaleVector(vecVeloc, speed);
	
	TeleportEntity(entity, origin, angRotation, vecVeloc);
	
	char _tmp[ 128 ];
	FormatEx(_tmp, sizeof(_tmp), "OnUser1 !self:kill::%0.4f:-1", GetVectorDistance(origin, dest) / speed);
	SetVariantString(_tmp);
	AcceptEntityInput(entity, "AddOutput");
	AcceptEntityInput(entity, "FireUser1");
	
	return entity;
}